/** 
 *  @file   FrameQ.xdc
 *
 *  @brief      XDC file
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */





package ti.syslink.ipc.rtos

import ti.sdo.ipc.ListMP ;
import xdc.runtime.Error;
import ti.sdo.utils.NameServer;
import xdc.runtime.IGateProvider;

@ModuleStartup
@InstanceInitError /* initialization mat throw errors */
@InstanceFinalize
module FrameQ inherits ti.syslink.ipc.rtos.interfaces.IFrameQ {
    /*!
     *  Assert raised when an invalid parameter(s) is passed into a function
     */
    config xdc.runtime.Assert.Id A_invalidParams =
        {msg: "A_invalidParams: Invalid parameter"};

    /*!
     *  Assert raised when a null pointer needs to be dereferenced
     */
    config xdc.runtime.Assert.Id A_nullPointer =
        {msg: "A_nullPointer: Null pointer"};

    /*!
     *  Assert raised when n invalid shared region address  specified.
     */
    config xdc.runtime.Assert.Id A_InvalidSharedRegionAddr =
        {msg: "A_InvalidSharedRegionAddr: Invalid shared region address"};

    /*!
     *  Assert raised when n invalid shared region address  specified.
     */
    config xdc.runtime.Assert.Id A_invalidFrameQBufMgrId =
        {msg: "A_invalidFrameQBufMgrId: Invalid FrameQBufMgr id specified"};

/*!
     *  Assert raised when n invalid shared region address  specified.
     */
    config xdc.runtime.Assert.Id A_memoryAllocFail =
        {msg: "A_memoryAllocFail: Memory_alloc failed"} ;

   /*!
     *  Assert raised when null handle is returned from  modules.
     */
    config xdc.runtime.Assert.Id A_nullHandle =
        {msg: "A_nullHandle: Null Handle"};

    /*!
     *  Error raised when Invalid args passed.
     */
    config Error.Id E_InvalidArg  = {
        msg: "E_InvalidArg : Invalid Arguments passed"
    };

    /*!
     *  Error raised when Memory alloc failed.
     */
    config Error.Id E_Memory  = {
        msg: "E_Memory  : Memory alloc failed"
    };

    /*!
     *  Error raised when FrameQ is not created.
     */
    config Error.Id E_FrameQNotCreated  = {
        msg: "E_FrameQNotCreated: FrameQ instance is not created"
    };

    /*!
     *  Error raised when FrameQ is not Found in NameServer.
     */
    config Error.Id E_FrameQNotFound  = {
        msg: "E_FrameQNotFound: FrameQ instance is not found in NameServer"
    };

    /*!
     *  Error raised when FrameQ is already created.
     */
    config Error.Id E_FrameQCreated = {
        msg: "E_FrameQCreated: FrameQ instance is already created"
    };

    /*!
     *  Error raised when Gate Create failed.
     */
    config Error.Id E_GateCreate  = {
        msg: "E_GateCreate: Gate instance Create Failed"
    };

    /*!
     *  Error raised when Gate open failed.
     */
    config Error.Id E_GateOpen  = {
        msg: "E_GateOpen: Gate instance Open Failed"
    };

    /*!
     *  Error raised when an invalid parameter(s) is passed into a function
     */
    config Error.Id E_InvalidParams =
        {msg: "E_InvalidParams: Invalid parameter"};

    /*!
     *  Error raised when an invalid interface type mentioned in params
     */
    config Error.Id E_InvalidInterfaceType =
        {msg: "E_InvalidInterfaceType: Invalid interface type"};


    /*!
     *  Error raised when an instance already exists
     */
    config Error.Id E_InstanceExists = {
        msg: "E_InstanceExists: Instance already exists"
    };

    /*!
     *  Error raised when  internal FrameQ create failed.
     */
    config Error.Id E_FrameQCreateFail  = {
        msg: "E_FrameQCreateFail: Internal FrameQ instance create API failed"
    };

    /*!
     *  Error raised when FrameQ open failed.
     */
    config Error.Id E_FrameQOpenFail  = {
        msg: "E_FrameQOpenFail: FrameQ instance open failed"
    };


    /*!
     *  Error raised when ClientNotifyMgr Create failed.
     */
    config Error.Id E_ClientNotifyMgrCreate  = {
        msg: "E_ClientNotifyMgrCreate: ClientNotifyMgr instance Create Failed"
    };

    /*!
     *  Error raised when ClientNotifyMgrOpen failed.
     */
    config Error.Id E_ClientNotifyMgrOpen  = {
        msg: "E_ClientNotifyMgrOpen: ClientNotifyMgrOpen instance Open Failed"
    };

    /*!
     *  Error raised when Insufficient shared memory provided to create call.
     */
    config Error.Id E_InvSize  = {
        msg: "E_InvSize: SharedMemory provided to create call is less than the instance shared memory requirements"
    };

    /*!
     *  Error raised when Insufficient shared addrprovided to create call.
     */
    config Error.Id E_InvAddr  = {
        msg: "E_InvAddr: Shared Addr provide to create call is NULL"
    };
    /*!
     *  Error raised when buffersize is not aligned to Cache line size.
     */
    config Error.Id E_CacheAlign  = {
        msg: "E_CacheAlign: BufferSize is not Cache Aligned"
    };


    /*! @_nodoc
     *  ========= ShMem_InstInfoParams ==========
     *  Structure defining Additional specific parameters that name server maintains
     *  along with the interface  type.These vary based on implementation.
     *
     *  @field(sharedAddr)     Shared Region Address  for control structures.
     *  @field(sharedAddrSize) Size of the shared memory  for control structure.
	 *  @field(instGateMPAddr)     Instance gateMP shared address in
	 *                             SharedRegion_SRPtr format.
     *  @field(frameQBufMgrShAddr) SharedAddr of the FrameQBufMgr that
     *                             FrameQ instance is using in
     *                             SharedRegion_SrPtr format. Not valid if
     *                             frameQBufMgrName is  valid.
     *  @field(frameQBufMgrName) Name of the FrameQBufMgr instance that
     *                           FrameQInstance is using.
     */
    struct ShMem_InstInfoParams  {
        Ptr        sharedAddr;
        UInt32     sharedAddrSize;
        Ptr        instGateMPAddr;
        Ptr        frameQBufMgrSharedAddr;
        UInt8      frameQBufMgrName[MAX_NAMELEN];
		Ptr        cliGateMPAddr;
    }

    /*! @_nodoc
     *  ======== NameServerEntry ========
     *  mouduleconfig structure.
     *
     *  @field(interfaceType)  type of the interface. see {@link Interface}.
     *  @field(instParams)  instparams  is  union of the implementation's
     *                      nameserver entry.
     *                      the individual nameserver entry contents may vary
     *                      based on implementation.
     */
    struct NameServerEntry {
        UInt32     ctrlInterfaceType;
        union {
            ShMem_InstInfoParams          shMemParams;
        } instParams;
    };


    /*! @_nodoc
     *  ======== isSupportedInterface ========
     *  API to find out  if the given interface type is supported.
     *
     *  @param(interfaceType)  type of the interface.
     *
     */
    Int32 isSupportedInterface(UInt32 interfaceType);

    /*! @_nodoc
     *  ======== OpenParams ========
     *  mouduleconfig structure.
     *
     *  @field(commonOpenParams)
     *                            Open params common to all the implementations.
     *
     */
    struct OpenParams {
        CommonOpenParams commonOpenParams;
    };

     instance:
        config CreateParams commonCreateParams = {size:0,
                                                  ctrlInterfaceType:INTERFACE_NONE,
                                                  name:null,
                                                  openFlag:false};
   /*!
     *  ======== impParams ========
     *  Pointer to implementation specific parameters
     *
     */
     config Ptr impParams = null;

   /*!
     *  ======== impParams ========
     *  Pointer to implementation specific parameters
     *
     */
     config UInt32  impParamsSize = 0;

     internal:
  /*!
     *  ======== Module_State ========
     *  Module  state Object structure.
     *
     *  @field(nameServer)        Handle to the local NameServer used for storing FrameQ instance information.
     *  @field(nameServerType)    Boolean flag denotes whether NameServer instance
     *                            is dynamically created or static.
     *  @field(cfg)               If module initialized cfg contains the
     *                            configured params of type {@link Config}.
     *  @field(refCount)          Refcount denoting module initialize count.
     */
    struct Module_State {
        NameServer.Handle     nameServer;
        UInt32                nameServerType;
        Config                cfg;
        UInt32                refCount;
    };

   /*!
     *  ======== Instance_State ========
     *  Instance  state Object structure.
     *
     *  @field(name)             Name of the instance.
     *  @field(gate)             Gate for critical region management.
     *  @field(objType)          Denotes type of object.Dynamic create or static create or Dynamic open or static open.
     *  @field(objMode)          Denotes mode of the object.Reader  or writer or none.
     *  @field(controlStruct)    Pointer to the shared instance control structure.
     *  @field(virtClientHandle) Pointer to the Client's shared  info. For reader this points to the {@link #ReaderClient}
     *                           For writer this points to the {@link #WriterClient}
     *
     */
    /* not for client use */
     struct Instance_State {
            String                  name ;
            UInt32                  objType ;
            UInt32                  objMode ;
            UInt32    ctrlInterfaceType;
            Ptr                     handle;
            Bool                    ctrlStructCacheFlag ;
            Bool                    frmHdrBufCacheFlag ;
            Bool                    frmBufCacheFlag[8] ;
            Bool                    bufCpuAccessFlag[8];

     };



}
