/** 
 *  @file   ISyslinkMemMgr.xdc
 *
 *  @brief      Interface file for syslink memory manager.
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */


package ti.syslink.ipc.rtos.interfaces;

import xdc.runtime.Assert;
import xdc.runtime.IHeap;
import ti.syslink.ipc.rtos.interfaces.SysLink ;


interface ISyslinkMemMgr {


/*    config Assert.Id A_nullArgument  = {
        msg: "A_nullArgument: Null argument passed"
    };*/

    const UInt8 MAX_NAME_LENGTH  = 32;
    const UInt8 MAX_TILER_BLOCKS = 10;

    enum Type{
        TYPE_SHAREDMEM = 0x0,
        TYPE_TILERMEM  = 0x1,
        TYPE_NONE      = 0x2
    };


    enum AddrType{
        AddrType_Virtual  = 0u,
        AddrType_Portable = 1u,
        AddrType_EndValue = 2u
    };

    struct AllocBlock {
        UInt16 pixelFormat;
        union {
            struct {
                UInt16 width;
                UInt16 height;
            } area;
            UInt16  len;
        } dim;
        UInt32 stride;
    };

    struct createParams {
        UInt32     size;
        Type       type;
        UInt8      name[MAX_NAME_LENGTH];
    };

    struct allocParams {
        UInt32 size;
        UInt32 align;
        AllocBlock block[MAX_TILER_BLOCKS];
        UInt32 tilerBlocks;
    };

    struct freeParams {
        Ptr    ptr;
        UInt32 size;
    };

    instance :

    config IHeap.Handle heapHandle = null;

    /* Function to allocate memory from the SyslinkMemMgr */
    Ptr alloc (Ptr allocParams);

    /* Function to allocate memory from the SyslinkMemMgr */
    Int32 free (Ptr freeParams);

    /* Function to map. not used for shared memory*/
    /* Ptr map (Ptr arg);*/

    /* Function to do unmap */
    /*Int32 unmap (Ptr arg);*/

    /* Function to translate source address to destination address type */
    Ptr translate (Ptr             srcAddr,
                   AddrType        srcAddrType,
                   AddrType        desAddrType);

}

